@@ -1,3 +1,5 @@
+// Copyright 2010-2019 Keith Goodman
+// Copyright 2019 Bottleneck Developers
 #include "bottleneck.h"
 #include "iterators.h"
 
@@ -11,6 +13,16 @@
     iter it; \
     init_iter_all(&it, a, 1, 0);
 
+#define INIT_ALL_RAVEL_ANY_ORDER \
+    iter it; \
+    init_iter_all(&it, a, 1, 1);
+
+/* used with INIT_ALL_RAVEL */
+#define DECREF_INIT_ALL_RAVEL \
+    if (it.a_ravel != NULL) { \
+        Py_DECREF(it.a_ravel); \
+    }
+
 #define INIT_ONE(dtype0, dtype1) \
     iter it; \
     PyObject *y; \
@@ -72,8 +84,7 @@ reducer(char *name,
 /* nansum ---------------------------------------------------------------- */
 
 /* dtype = [['float64'], ['float32']] */
-REDUCE_ALL(nansum, DTYPE0)
-{
+REDUCE_ALL(nansum, DTYPE0) {
     npy_DTYPE0 ai, asum = 0;
     INIT_ALL
     BN_BEGIN_ALLOW_THREADS
@@ -88,15 +99,13 @@ REDUCE_ALL(nansum, DTYPE0)
     return PyFloat_FromDouble(asum);
 }
 
-REDUCE_ONE(nansum, DTYPE0)
-{
+REDUCE_ONE(nansum, DTYPE0) {
     npy_DTYPE0 ai, asum;
     INIT_ONE(DTYPE0, DTYPE0)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(0)
-    }
-    else {
+    } else {
         WHILE {
             asum = 0;
             FOR {
@@ -113,8 +122,7 @@ REDUCE_ONE(nansum, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64'], ['int32']] */
-REDUCE_ALL(nansum, DTYPE0)
-{
+REDUCE_ALL(nansum, DTYPE0) {
     npy_DTYPE0 asum = 0;
     INIT_ALL
     BN_BEGIN_ALLOW_THREADS
@@ -123,18 +131,16 @@ REDUCE_ALL(nansum, DTYPE0)
         NEXT
     }
     BN_END_ALLOW_THREADS
-    return PyInt_FromLong(asum);
+    return PyLong_FromLongLong(asum);
 }
 
-REDUCE_ONE(nansum, DTYPE0)
-{
+REDUCE_ONE(nansum, DTYPE0) {
     npy_DTYPE0 asum;
     INIT_ONE(DTYPE0, DTYPE0)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(0)
-    }
-    else {
+    } else {
         WHILE {
             asum = 0;
             FOR asum += AI(DTYPE0);
@@ -153,8 +159,7 @@ REDUCE_MAIN(nansum, 0)
 /* nanmean ---------------------------------------------------------------- */
 
 /* dtype = [['float64'], ['float32']] */
-REDUCE_ALL(nanmean, DTYPE0)
-{
+REDUCE_ALL(nanmean, DTYPE0) {
     Py_ssize_t count = 0;
     npy_DTYPE0 ai, asum = 0;
     INIT_ALL
@@ -177,18 +182,17 @@ REDUCE_ALL(nanmean, DTYPE0)
     }
 }
 
-REDUCE_ONE(nanmean, DTYPE0)
-{
+REDUCE_ONE(nanmean, DTYPE0) {
     Py_ssize_t count;
     npy_DTYPE0 ai, asum;
     INIT_ONE(DTYPE0, DTYPE0)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(BN_NAN)
-    }
-    else {
+    } else {
         WHILE {
-            asum = count = 0;
+            count = 0;
+            asum = 0;
             FOR {
                 ai = AI(DTYPE0);
                 if (ai == ai) {
@@ -211,8 +215,7 @@ REDUCE_ONE(nanmean, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64', 'float64'], ['int32', 'float64']] */
-REDUCE_ALL(nanmean, DTYPE0)
-{
+REDUCE_ALL(nanmean, DTYPE0) {
     Py_ssize_t total_length = 0;
     npy_DTYPE1 asum = 0;
     INIT_ALL
@@ -230,15 +233,13 @@ REDUCE_ALL(nanmean, DTYPE0)
     }
 }
 
-REDUCE_ONE(nanmean, DTYPE0)
-{
+REDUCE_ONE(nanmean, DTYPE0) {
     npy_DTYPE1 asum;
     INIT_ONE(DTYPE1, DTYPE1)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(BN_NAN)
-    }
-    else {
+    } else {
         WHILE {
             asum = 0;
             FOR asum += AI(DTYPE0);
@@ -264,8 +265,7 @@ REDUCE_MAIN(nanmean, 0)
 /* repeat = {'NAME': ['nanstd', 'nanvar'],
              'FUNC': ['sqrt',   '']} */
 /* dtype = [['float64'], ['float32']] */
-REDUCE_ALL(NAME, DTYPE0)
-{
+REDUCE_ALL(NAME, DTYPE0) {
     Py_ssize_t count = 0;
     npy_DTYPE0 ai, amean, out, asum = 0;
     INIT_ALL
@@ -295,26 +295,24 @@ REDUCE_ALL(NAME, DTYPE0)
             NEXT
         }
         out = FUNC(asum / (count - ddof));
-    }
-    else {
+    } else {
         out = BN_NAN;
     }
     BN_END_ALLOW_THREADS
     return PyFloat_FromDouble(out);
 }
 
-REDUCE_ONE(NAME, DTYPE0)
-{
+REDUCE_ONE(NAME, DTYPE0) {
     Py_ssize_t count;
     npy_DTYPE0 ai, asum, amean;
     INIT_ONE(DTYPE0, DTYPE0)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(BN_NAN)
-    }
-    else {
+    } else {
         WHILE {
-            asum = count = 0;
+            count = 0;
+            asum = 0;
             FOR {
                 ai = AI(DTYPE0);
                 if (ai == ai) {
@@ -333,8 +331,7 @@ REDUCE_ONE(NAME, DTYPE0)
                     }
                 }
                 asum = FUNC(asum / (count - ddof));
-            }
-            else {
+            } else {
                 asum = BN_NAN;
             }
             YPP = asum;
@@ -347,8 +344,7 @@ REDUCE_ONE(NAME, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64', 'float64'], ['int32', 'float64']] */
-REDUCE_ALL(NAME, DTYPE0)
-{
+REDUCE_ALL(NAME, DTYPE0) {
     npy_DTYPE1 out;
     Py_ssize_t size = 0;
     npy_DTYPE1 ai, amean, asum = 0;
@@ -371,16 +367,14 @@ REDUCE_ALL(NAME, DTYPE0)
             NEXT
         }
         out = FUNC(asum / (size - ddof));
-    }
-    else {
+    } else {
         out = BN_NAN;
     }
     BN_END_ALLOW_THREADS
     return PyFloat_FromDouble(out);
 }
 
-REDUCE_ONE(NAME, DTYPE0)
-{
+REDUCE_ONE(NAME, DTYPE0) {
     npy_DTYPE1 ai, asum, amean, length_inv, length_ddof_inv;
     INIT_ONE(DTYPE1, DTYPE1)
     BN_BEGIN_ALLOW_THREADS
@@ -388,8 +382,7 @@ REDUCE_ONE(NAME, DTYPE0)
     length_ddof_inv = 1.0 / (LENGTH - ddof);
     if (LENGTH == 0) {
         FILL_Y(BN_NAN)
-    }
-    else {
+    } else {
         WHILE {
             asum = 0;
             FOR asum += AI(DTYPE0);
@@ -401,8 +394,7 @@ REDUCE_ONE(NAME, DTYPE0)
                     asum += ai * ai;
                 }
                 asum = FUNC(asum * length_ddof_inv);
-            }
-            else {
+            } else {
                 asum = BN_NAN;
             }
             YPP = asum;
@@ -425,8 +417,7 @@ REDUCE_MAIN(NAME, 1)
              'BIG_FLOAT': ['BN_INFINITY',    '-BN_INFINITY'],
              'BIG_INT':   ['NPY_MAX_DTYPE0', 'NPY_MIN_DTYPE0']} */
 /* dtype = [['float64'], ['float32']] */
-REDUCE_ALL(NAME, DTYPE0)
-{
+REDUCE_ALL(NAME, DTYPE0) {
     npy_DTYPE0 ai, extreme = BIG_FLOAT;
     int allnan = 1;
     INIT_ALL
@@ -451,8 +442,7 @@ REDUCE_ALL(NAME, DTYPE0)
     return PyFloat_FromDouble(extreme);
 }
 
-REDUCE_ONE(NAME, DTYPE0)
-{
+REDUCE_ONE(NAME, DTYPE0) {
     npy_DTYPE0 ai, extreme;
     int allnan;
     INIT_ONE(DTYPE0, DTYPE0)
@@ -482,8 +472,7 @@ REDUCE_ONE(NAME, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64'], ['int32']] */
-REDUCE_ALL(NAME, DTYPE0)
-{
+REDUCE_ALL(NAME, DTYPE0) {
     npy_DTYPE0 ai, extreme = BIG_INT;
     INIT_ALL
     if (SIZE == 0) {
@@ -500,11 +489,10 @@ REDUCE_ALL(NAME, DTYPE0)
         NEXT
     }
     BN_END_ALLOW_THREADS
-    return PyInt_FromLong(extreme);
+    return PyLong_FromLongLong(extreme);
 }
 
-REDUCE_ONE(NAME, DTYPE0)
-{
+REDUCE_ONE(NAME, DTYPE0) {
     npy_DTYPE0 ai, extreme;
     INIT_ONE(DTYPE0, DTYPE0)
     if (LENGTH == 0) {
@@ -538,13 +526,13 @@ REDUCE_MAIN(NAME, 0)
              'BIG_FLOAT': ['BN_INFINITY',    '-BN_INFINITY'],
              'BIG_INT':   ['NPY_MAX_DTYPE0', 'NPY_MIN_DTYPE0']} */
 /* dtype = [['float64'], ['float32']] */
-REDUCE_ALL(NAME, DTYPE0)
-{
+REDUCE_ALL(NAME, DTYPE0) {
     npy_DTYPE0 ai, extreme = BIG_FLOAT;
     int allnan = 1;
     Py_ssize_t idx = 0;
     INIT_ALL_RAVEL
     if (SIZE == 0) {
+        DECREF_INIT_ALL_RAVEL
         VALUE_ERR("numpy.NAME raises on a.size==0 and axis=None; "
                   "So Bottleneck too.");
         return NULL;
@@ -559,16 +547,16 @@ REDUCE_ALL(NAME, DTYPE0)
         }
     }
     BN_END_ALLOW_THREADS
+    DECREF_INIT_ALL_RAVEL
     if (allnan) {
         VALUE_ERR("All-NaN slice encountered");
         return NULL;
     } else {
-        return PyInt_FromLong(idx);
+        return PyLong_FromLongLong(idx);
     }
 }
 
-REDUCE_ONE(NAME, DTYPE0)
-{
+REDUCE_ONE(NAME, DTYPE0) {
     int allnan, err_code = 0;
     Py_ssize_t idx = 0;
     npy_DTYPE0 ai, extreme;
@@ -607,12 +595,12 @@ REDUCE_ONE(NAME, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64', 'intp'], ['int32', 'intp']] */
-REDUCE_ALL(NAME, DTYPE0)
-{
+REDUCE_ALL(NAME, DTYPE0) {
     npy_DTYPE1 idx = 0;
     npy_DTYPE0 ai, extreme = BIG_INT;
     INIT_ALL_RAVEL
     if (SIZE == 0) {
+        DECREF_INIT_ALL_RAVEL
         VALUE_ERR("numpy.NAME raises on a.size==0 and axis=None; "
                   "So Bottleneck too.");
         return NULL;
@@ -626,11 +614,11 @@ REDUCE_ALL(NAME, DTYPE0)
         }
     }
     BN_END_ALLOW_THREADS
-    return PyInt_FromLong(idx);
+    DECREF_INIT_ALL_RAVEL
+    return PyLong_FromLongLong(idx);
 }
 
-REDUCE_ONE(NAME, DTYPE0)
-{
+REDUCE_ONE(NAME, DTYPE0) {
     npy_DTYPE1 idx = 0;
     npy_DTYPE0 ai, extreme;
     INIT_ONE(DTYPE1, DTYPE1)
@@ -642,7 +630,7 @@ REDUCE_ONE(NAME, DTYPE0)
     BN_BEGIN_ALLOW_THREADS
     WHILE {
         extreme = BIG_INT;
-        FOR_REVERSE{
+        FOR_REVERSE {
             ai = AI(DTYPE0);
             if (ai COMPARE extreme) {
                 extreme = ai;
@@ -664,8 +652,7 @@ REDUCE_MAIN(NAME, 0)
 /* ss ---------------------------------------------------------------- */
 
 /* dtype = [['float64'], ['float32']] */
-REDUCE_ALL(ss, DTYPE0)
-{
+REDUCE_ALL(ss, DTYPE0) {
     npy_DTYPE0 ai, asum = 0;
     INIT_ALL
     BN_BEGIN_ALLOW_THREADS
@@ -680,18 +667,16 @@ REDUCE_ALL(ss, DTYPE0)
     return PyFloat_FromDouble(asum);
 }
 
-REDUCE_ONE(ss, DTYPE0)
-{
+REDUCE_ONE(ss, DTYPE0) {
     npy_DTYPE0 ai, asum;
     INIT_ONE(DTYPE0, DTYPE0)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(0)
-    }
-    else {
+    } else {
         WHILE {
             asum = 0;
-            FOR{
+            FOR {
                 ai = AI(DTYPE0);
                 asum += ai * ai;
             }
@@ -705,8 +690,7 @@ REDUCE_ONE(ss, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64'], ['int32']] */
-REDUCE_ALL(ss, DTYPE0)
-{
+REDUCE_ALL(ss, DTYPE0) {
     npy_DTYPE0 ai, asum = 0;
     INIT_ALL
     BN_BEGIN_ALLOW_THREADS
@@ -718,21 +702,19 @@ REDUCE_ALL(ss, DTYPE0)
         NEXT
     }
     BN_END_ALLOW_THREADS
-    return PyInt_FromLong(asum);
+    return PyLong_FromLongLong(asum);
 }
 
-REDUCE_ONE(ss, DTYPE0)
-{
+REDUCE_ONE(ss, DTYPE0) {
     npy_DTYPE0 ai, asum;
     INIT_ONE(DTYPE0, DTYPE0)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(0)
-    }
-    else {
+    } else {
         WHILE {
             asum = 0;
-            FOR{
+            FOR {
                 ai = AI(DTYPE0);
                 asum += ai * ai;
             }
@@ -760,8 +742,7 @@ REDUCE_MAIN(ss, 0)
             if (ai > amax) amax = ai; \
         } \
         med = 0.5 * (B(dtype, k) + amax); \
-    } \
-    else { \
+    } else { \
         med =  B(dtype, k); \
     } \
 
@@ -828,35 +809,33 @@ REDUCE_MAIN(ss, 0)
 /* repeat = {'NAME': ['median', 'nanmedian'],
              'FUNC': ['MEDIAN', 'NANMEDIAN']} */
 /* dtype = [['float64', 'float64'], ['float32', 'float32']] */
-REDUCE_ALL(NAME, DTYPE0)
-{
+
+REDUCE_ALL(NAME, DTYPE0) {
     npy_intp i;
     npy_DTYPE1 med;
-    INIT_ALL_RAVEL
+    INIT_ALL_RAVEL_ANY_ORDER
     BN_BEGIN_ALLOW_THREADS
     BUFFER_NEW(DTYPE0, LENGTH)
     if (LENGTH == 0) {
         med = BN_NAN;
-    }
-    else {
+    } else {
         FUNC(DTYPE0)
     }
     done:
     BUFFER_DELETE
     BN_END_ALLOW_THREADS
+    DECREF_INIT_ALL_RAVEL
     return PyFloat_FromDouble(med);
 }
 
-REDUCE_ONE(NAME, DTYPE0)
-{
+REDUCE_ONE(NAME, DTYPE0) {
     npy_intp i;
     npy_DTYPE1 med;
     INIT_ONE(DTYPE1, DTYPE1)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(BN_NAN)
-    }
-    else {
+    } else {
         BUFFER_NEW(DTYPE0, LENGTH)
         WHILE {
             FUNC(DTYPE0)
@@ -873,34 +852,31 @@ REDUCE_ONE(NAME, DTYPE0)
 /* repeat end */
 
 /* dtype = [['int64', 'float64'], ['int32', 'float64']] */
-REDUCE_ALL(median, DTYPE0)
-{
+REDUCE_ALL(median, DTYPE0) {
     npy_intp i;
     npy_DTYPE1 med;
-    INIT_ALL_RAVEL
+    INIT_ALL_RAVEL_ANY_ORDER
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         med = BN_NAN;
-    }
-    else {
+    } else {
         BUFFER_NEW(DTYPE0, LENGTH)
         MEDIAN_INT(DTYPE0)
         BUFFER_DELETE
     }
     BN_END_ALLOW_THREADS
+    DECREF_INIT_ALL_RAVEL
     return PyFloat_FromDouble(med);
 }
 
-REDUCE_ONE(median, DTYPE0)
-{
+REDUCE_ONE(median, DTYPE0) {
     npy_intp i;
     npy_DTYPE1 med;
     INIT_ONE(DTYPE1, DTYPE1)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(BN_NAN)
-    }
-    else {
+    } else {
         BUFFER_NEW(DTYPE0, LENGTH)
         WHILE {
             MEDIAN_INT(DTYPE0)
@@ -917,8 +893,7 @@ REDUCE_ONE(median, DTYPE0)
 REDUCE_MAIN(median, 0)
 
 static PyObject *
-nanmedian(PyObject *self, PyObject *args, PyObject *kwds)
-{
+nanmedian(PyObject *self, PyObject *args, PyObject *kwds) {
     return reducer("nanmedian",
                    args,
                    kwds,
@@ -936,8 +911,7 @@ nanmedian(PyObject *self, PyObject *args, PyObject *kwds)
 /* anynan ---------------------------------------------------------------- */
 
 /* dtype = [['float64'], ['float32']] */
-REDUCE_ALL(anynan, DTYPE0)
-{
+REDUCE_ALL(anynan, DTYPE0) {
     int f = 0;
     npy_DTYPE0 ai;
     INIT_ALL
@@ -958,16 +932,14 @@ REDUCE_ALL(anynan, DTYPE0)
     Py_RETURN_FALSE;
 }
 
-REDUCE_ONE(anynan, DTYPE0)
-{
+REDUCE_ONE(anynan, DTYPE0) {
     int f;
     npy_DTYPE0 ai;
     INIT_ONE(BOOL, uint8)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(0)
-    }
-    else {
+    } else {
         WHILE {
             f = 0;
             FOR {
@@ -987,13 +959,13 @@ REDUCE_ONE(anynan, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64'], ['int32']] */
-REDUCE_ALL(anynan, DTYPE0)
-{
+BN_OPT_3
+REDUCE_ALL(anynan, DTYPE0) {
     Py_RETURN_FALSE;
 }
 
-REDUCE_ONE(anynan, DTYPE0)
-{
+BN_OPT_3
+REDUCE_ONE(anynan, DTYPE0) {
     INIT_ONE(BOOL, uint8)
     BN_BEGIN_ALLOW_THREADS
     FILL_Y(0);
@@ -1008,8 +980,7 @@ REDUCE_MAIN(anynan, 0)
 /* allnan ---------------------------------------------------------------- */
 
 /* dtype = [['float64'], ['float32']] */
-REDUCE_ALL(allnan, DTYPE0)
-{
+REDUCE_ALL(allnan, DTYPE0) {
     int f = 0;
     npy_DTYPE0 ai;
     INIT_ALL
@@ -1030,16 +1001,14 @@ REDUCE_ALL(allnan, DTYPE0)
     Py_RETURN_TRUE;
 }
 
-REDUCE_ONE(allnan, DTYPE0)
-{
+REDUCE_ONE(allnan, DTYPE0) {
     int f;
     npy_DTYPE0 ai;
     INIT_ONE(BOOL, uint8)
     BN_BEGIN_ALLOW_THREADS
     if (LENGTH == 0) {
         FILL_Y(1)
-    }
-    else {
+    } else {
         WHILE {
             f = 1;
             FOR {
@@ -1059,20 +1028,19 @@ REDUCE_ONE(allnan, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64'], ['int32']] */
-REDUCE_ALL(allnan, DTYPE0)
-{
+BN_OPT_3
+REDUCE_ALL(allnan, DTYPE0) {
     if (PyArray_SIZE(a) == 0) Py_RETURN_TRUE;
     Py_RETURN_FALSE;
 }
 
-REDUCE_ONE(allnan, DTYPE0)
-{
+BN_OPT_3
+REDUCE_ONE(allnan, DTYPE0) {
     INIT_ONE(BOOL, uint8)
     BN_BEGIN_ALLOW_THREADS
     if (SIZE == 0) {
         FILL_Y(1);
-    }
-    else {
+    } else {
         FILL_Y(0);
     }
     BN_END_ALLOW_THREADS
@@ -1091,22 +1059,21 @@ PyObject *pystr_ddof = NULL;
 
 static int
 intern_strings(void) {
-    pystr_a = PyString_InternFromString("a");
-    pystr_axis = PyString_InternFromString("axis");
-    pystr_ddof = PyString_InternFromString("ddof");
+    pystr_a = PyUnicode_InternFromString("a");
+    pystr_axis = PyUnicode_InternFromString("axis");
+    pystr_ddof = PyUnicode_InternFromString("ddof");
     return pystr_a && pystr_axis && pystr_ddof;
 }
 
 /* reducer --------------------------------------------------------------- */
 
-static BN_INLINE int
+static inline int
 parse_args(PyObject *args,
            PyObject *kwds,
            int has_ddof,
            PyObject **a,
            PyObject **axis,
-           PyObject **ddof)
-{
+           PyObject **ddof) {
     const Py_ssize_t nargs = PyTuple_GET_SIZE(args);
     const Py_ssize_t nkwds = kwds == NULL ? 0 : PyDict_Size(kwds);
     if (nkwds) {
@@ -1162,8 +1129,7 @@ parse_args(PyObject *args,
             TYPE_ERR("too many arguments");
             return 0;
         }
-    }
-    else {
+    } else {
         switch (nargs) {
             case 3:
                 if (has_ddof) {
@@ -1199,16 +1165,16 @@ reducer(char *name,
         fone_t fone_float32,
         fone_t fone_int64,
         fone_t fone_int32,
-        int has_ddof)
-{
+        int has_ddof) {
 
     int ndim;
-    int axis;
+    int axis = 0; /* initialize to avoid compiler error */
     int dtype;
     int ddof;
     int reduce_all = 0;
 
     PyArrayObject *a;
+    PyObject *y;
 
     PyObject *a_obj = NULL;
     PyObject *axis_obj = Py_None;
@@ -1219,8 +1185,9 @@ reducer(char *name,
     }
 
     /* convert to array if necessary */
-    if PyArray_Check(a_obj) {
+    if (PyArray_Check(a_obj)) {
         a = (PyArrayObject *)a_obj;
+        Py_INCREF(a);
     } else {
         a = (PyArrayObject *)PyArray_FROM_O(a_obj);
         if (a == NULL) {
@@ -1229,19 +1196,19 @@ reducer(char *name,
     }
 
     /* check for byte swapped input array */
-    if PyArray_ISBYTESWAPPED(a) {
+    if (PyArray_ISBYTESWAPPED(a)) {
+        Py_DECREF(a);
         return slow(name, args, kwds);
     }
 
     /* does user want to reduce over all axes? */
     if (axis_obj == Py_None) {
         reduce_all = 1;
-    }
-    else {
+    } else {
         axis = PyArray_PyIntAsInt(axis_obj);
         if (error_converting(axis)) {
             TYPE_ERR("`axis` must be an integer or None");
-            return NULL;
+            goto error;
         }
         ndim = PyArray_NDIM(a);
         if (axis < 0) {
@@ -1249,12 +1216,11 @@ reducer(char *name,
             if (axis < 0) {
                 PyErr_Format(PyExc_ValueError,
                              "axis(=%d) out of bounds", axis);
-                return NULL;
+                goto error;
             }
-        }
-        else if (axis >= ndim) {
+        } else if (axis >= ndim) {
             PyErr_Format(PyExc_ValueError, "axis(=%d) out of bounds", axis);
-            return NULL;
+            goto error;
         }
         if (ndim == 1) {
             reduce_all = 1;
@@ -1264,12 +1230,11 @@ reducer(char *name,
     /* ddof */
     if (ddof_obj == NULL) {
         ddof = 0;
-    }
-    else {
+    } else {
         ddof = PyArray_PyIntAsInt(ddof_obj);
         if (error_converting(ddof)) {
             TYPE_ERR("`ddof` must be an integer");
-            return NULL;
+            goto error;
         }
     }
 
@@ -1278,41 +1243,40 @@ reducer(char *name,
     if (reduce_all == 1) {
         /* we are reducing the array along all axes */
         if (dtype == NPY_FLOAT64) {
-            return fall_float64(a, ddof);
+            y = fall_float64(a, ddof);
+        } else if (dtype == NPY_FLOAT32) {
+            y = fall_float32(a, ddof);
+        } else if (dtype == NPY_INT64) {
+            y = fall_int64(a, ddof);
+        } else if (dtype == NPY_INT32) {
+            y = fall_int32(a, ddof);
+        } else {
+            y = slow(name, args, kwds);
         }
-        else if (dtype == NPY_FLOAT32) {
-            return fall_float32(a, ddof);
-        }
-        else if (dtype == NPY_INT64) {
-            return fall_int64(a, ddof);
-        }
-        else if (dtype == NPY_INT32) {
-            return fall_int32(a, ddof);
-        }
-        else {
-            return slow(name, args, kwds);
-        }
-    }
-    else {
+    } else {
         /* we are reducing an array with ndim > 1 over a single axis */
         if (dtype == NPY_FLOAT64) {
-            return fone_float64(a, axis, ddof);
-        }
-        else if (dtype == NPY_FLOAT32) {
-            return fone_float32(a, axis, ddof);
-        }
-        else if (dtype == NPY_INT64) {
-            return fone_int64(a, axis, ddof);
-        }
-        else if (dtype == NPY_INT32) {
-            return fone_int32(a, axis, ddof);
-        }
-        else {
-            return slow(name, args, kwds);
+            y = fone_float64(a, axis, ddof);
+        } else if (dtype == NPY_FLOAT32) {
+            y = fone_float32(a, axis, ddof);
+        } else if (dtype == NPY_INT64) {
+            y = fone_int64(a, axis, ddof);
+        } else if (dtype == NPY_INT32) {
+            y = fone_int32(a, axis, ddof);
+        } else {
+            y = slow(name, args, kwds);
         }
 
     }
 
+    Py_DECREF(a);
+
+    return y;
+
+error:
+    Py_DECREF(a);
+    return NULL;
+
 }
 
 /* docstrings ------------------------------------------------------------- */
@@ -1987,7 +1951,6 @@ reduce_methods[] = {
 };
 
 
-#if PY_MAJOR_VERSION >= 3
 static struct PyModuleDef
 reduce_def = {
    PyModuleDef_HEAD_INIT,
@@ -1996,27 +1959,21 @@ reduce_def = {
    -1,
    reduce_methods
 };
-#endif
 
 
 PyMODINIT_FUNC
-#if PY_MAJOR_VERSION >= 3
-#define RETVAL m
 PyInit_reduce(void)
-#else
-#define RETVAL
-initreduce(void)
-#endif
 {
-    #if PY_MAJOR_VERSION >=3
-        PyObject *m = PyModule_Create(&reduce_def);
-    #else
-        PyObject *m = Py_InitModule3("reduce", reduce_methods, reduce_doc);
+    PyObject *m = PyModule_Create(&reduce_def);
+    if (m == NULL) return NULL;
+
+    #ifdef Py_GIL_DISABLED
+        PyUnstable_Module_SetGIL(m, Py_MOD_GIL_NOT_USED);
     #endif
-    if (m == NULL) return RETVAL;
+
     import_array();
     if (!intern_strings()) {
-        return RETVAL;
+        return NULL;
     }
-    return RETVAL;
+    return m;
 }
